"""
Collection of higher-level functions acting on varrays.
Mainly for general preprocessing.

Includes:

    COMPOSER -- phase match with short echo time reference scan

"""
#TODO
    #T2fit
    #dixon
    #etc

from .composer import *
from .wasabi import *
from .base import *

