import vnmrjpy as vj
import numpy as np
import copy

"""
Basic functions on varrays.
Includes:
    noise_mask
    average
    
"""

def mask(varr, threshold=None):
    """Create mask on image: set voxel data to 0 where image is below threshold.

    Noise threshold is assumed if set to None

    """ 
    return varr

def average(*varr_list):

    pass
